@@ -39,7 +39,7 @@
 
 static const ap_slotmem_provider_t *storage = NULL;
 static ap_slotmem_instance_t *slotmem = NULL;
-static int maxworkers = 0;
+static int maxworkers = 10;
 
 module AP_MODULE_DECLARE_DATA heartmonitor_module;
 
@@ -155,7 +155,7 @@ static void qs_to_table(const char *input, apr_table_t *parms,
         ap_unescape_url(value);
         apr_table_set(parms, key, value);
         /*
-           ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+           ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, APLOGNO(03182)
            "Found query arg: %s = %s", key, value);
          */
         key = apr_strtok(NULL, "&", &strtok_state);
@@ -171,7 +171,7 @@ static apr_status_t hm_update(void* mem, void *data, apr_pool_t *p)
     hm_slot_server_t *old = (hm_slot_server_t *) mem;
     hm_slot_server_ctx_t *s = (hm_slot_server_ctx_t *) data;
     hm_server_t *new = s->s;
-    if (strncmp(old->ip, new->ip, MAXIPSIZE)==0) {
+    if (strcmp(old->ip, new->ip)==0) {
         s->found = 1;
         old->busy = new->busy;
         old->ready = new->ready;
@@ -185,7 +185,7 @@ static apr_status_t hm_readid(void* mem, void *data, apr_pool_t *p)
     hm_slot_server_t *old = (hm_slot_server_t *) mem;
     hm_slot_server_ctx_t *s = (hm_slot_server_ctx_t *) data;
     hm_server_t *new = s->s;
-    if (strncmp(old->ip, new->ip, MAXIPSIZE)==0) {
+    if (strcmp(old->ip, new->ip)==0) {
         s->found = 1;
         s->item_id = old->id;
     }
@@ -202,7 +202,8 @@ static  apr_status_t  hm_slotmem_update_stat(hm_server_t *s, apr_pool_t *pool)
     if (!ctx.found) {
         unsigned int i;
         hm_slot_server_t hmserver;
-        memcpy(hmserver.ip, s->ip, MAXIPSIZE);
+        memset(&hmserver, 0, sizeof(hmserver));
+        apr_cpystrn(hmserver.ip, s->ip, sizeof(hmserver.ip));
         hmserver.busy = s->busy;
         hmserver.ready = s->ready;
         hmserver.seen = s->seen;
@@ -272,6 +273,7 @@ static apr_status_t hm_file_update_stat(hm_ctx_t *ctx, hm_server_t *s, apr_pool_
             char buf[4096];
             const char *ip;
             apr_size_t bsize = sizeof(buf);
+
             apr_brigade_cleanup(tmpbb);
             if (APR_BRIGADE_EMPTY(bb)) {
                 break;
@@ -292,46 +294,56 @@ static apr_status_t hm_file_update_stat(hm_ctx_t *ctx, hm_server_t *s, apr_pool_
             buf[bsize - 1] = 0;
             t = strchr(buf, ' ');
             if (t) {
-                ip = apr_pstrndup(pool, buf, t - buf);
-            } else {
+                ip = apr_pstrmemdup(pool, buf, t - buf);
+            }
+            else {
                 ip = NULL;
             }
+
             if (!ip || buf[0] == '#') {
                 /* copy things we can't process */
                 apr_file_printf(fp, "%s\n", buf);
-            } else if (strcmp(ip, s->ip) !=0 ) {
+            }
+            else if (strcmp(ip, s->ip) != 0 ) {
                 hm_server_t node;
                 apr_time_t seen;
+                const char *val;
+
                 /* Update seen time according to the last file modification */
                 apr_table_clear(hbt);
                 qs_to_table(apr_pstrdup(pool, t), hbt, pool);
-                if (apr_table_get(hbt, "busy")) {
-                    node.busy = atoi(apr_table_get(hbt, "busy"));
-                } else {
+                if ((val = apr_table_get(hbt, "busy"))) {
+                    node.busy = atoi(val);
+                }
+                else {
                     node.busy = 0;
                 }
 
-                if (apr_table_get(hbt, "ready")) {
-                    node.ready = atoi(apr_table_get(hbt, "ready"));
-                } else {
+                if ((val = apr_table_get(hbt, "ready"))) {
+                    node.ready = atoi(val);
+                }
+                else {
                     node.ready = 0;
                 }
 
-                if (apr_table_get(hbt, "lastseen")) {
-                    node.seen = atoi(apr_table_get(hbt, "lastseen"));
-                } else {
+                if ((val = apr_table_get(hbt, "lastseen"))) {
+                    node.seen = atoi(val);
+                }
+                else {
                     node.seen = SEEN_TIMEOUT;
                 }
                 seen = fage + node.seen;
 
-                if (apr_table_get(hbt, "port")) {
-                    node.port = atoi(apr_table_get(hbt, "port"));
-                } else {
+                if ((val = apr_table_get(hbt, "port"))) {
+                    node.port = atoi(val);
+                }
+                else {
                     node.port = 80;
                 }
                 apr_file_printf(fp, "%s &ready=%u&busy=%u&lastseen=%u&port=%u\n",
                                 ip, node.ready, node.busy, (unsigned int) seen, node.port);
-            } else {
+            }
+            else {
                 apr_time_t seen;
                 seen = apr_time_sec(now - s->seen);
                 apr_file_printf(fp, "%s &ready=%u&busy=%u&lastseen=%u&port=%u\n",
@@ -517,7 +529,7 @@ static hm_server_t *hm_get_server(hm_ctx_t *ctx, const char *ip, const int port)
 
 /* Process a message received from a backend node */
 static void hm_processmsg(hm_ctx_t *ctx, apr_pool_t *p,
-                                  apr_sockaddr_t *from, char *buf, int len)
+                          apr_sockaddr_t *from, char *buf, apr_size_t len)
 {
     apr_table_t *tbl;
 
@@ -613,9 +625,7 @@ static apr_status_t hm_watchdog_callback(int state, void *data,
             /* store in the slotmem or in the file depending on configuration */
             hm_update_stats(ctx, pool);
             cur = now = apr_time_sec(apr_time_now());
-            /* TODO: Insted HN_UPDATE_SEC use
-             * the ctx->interval
-             */
+
             while ((now - cur) < apr_time_sec(ctx->interval)) {
                 int n;
                 apr_status_t rc;
@@ -624,6 +634,7 @@ static apr_status_t hm_watchdog_callback(int state, void *data,
                 apr_interval_time_t timeout;
 
                 apr_pool_create(&p, pool);
+                apr_pool_tag(p, "hm_running");
 
                 pfd.desc_type = APR_POLL_SOCKET;
                 pfd.desc.s = ctx->sock;
@@ -753,7 +764,7 @@ static int hm_handler(request_rec *r)
     input_brigade = apr_brigade_create(r->connection->pool, r->connection->bucket_alloc);
     status = ap_get_brigade(r->input_filters, input_brigade, AP_MODE_READBYTES, APR_BLOCK_READ, MAX_MSG_LEN);
     if (status != APR_SUCCESS) {
-        return HTTP_INTERNAL_SERVER_ERROR;
+        return ap_map_http_request_error(status, HTTP_BAD_REQUEST);
     }
     apr_brigade_flatten(input_brigade, buf, &len);
 
@@ -792,12 +803,13 @@ static void *hm_create_config(apr_pool_t *p, server_rec *s)
     hm_ctx_t *ctx = (hm_ctx_t *) apr_palloc(p, sizeof(hm_ctx_t));
 
     ctx->active = 0;
-    ctx->storage_path = ap_server_root_relative(p, "logs/hb.dat");
+    ctx->storage_path = ap_runtime_dir_relative(p, DEFAULT_HEARTBEAT_STORAGE);
     /* TODO: Add directive for tuning the update interval
      */
     ctx->interval = apr_time_from_sec(HM_UPDATE_SEC);
     ctx->s = s;
     apr_pool_create(&ctx->p, p);
+    apr_pool_tag(ctx->p, "hm_ctx");
     ctx->servers = apr_hash_make(ctx->p);
 
     return ctx;
@@ -816,7 +828,7 @@ static const char *cmd_hm_storage(cmd_parms *cmd,
         return err;
     }
 
-    ctx->storage_path = ap_server_root_relative(p, path);
+    ctx->storage_path = ap_runtime_dir_relative(p, path);
 
     return NULL;
 }
