@@ -75,26 +75,6 @@ size_t h2_util_hex_dump(char *buffer, size_t maxlen,
     return strlen(buffer);
 }
 
-size_t h2_util_header_print(char *buffer, size_t maxlen,
-                            const char *name, size_t namelen,
-                            const char *value, size_t valuelen)
-{
-    size_t offset = 0;
-    size_t i;
-    for (i = 0; i < namelen && offset < maxlen; ++i, ++offset) {
-        buffer[offset] = name[i];
-    }
-    for (i = 0; i < 2 && offset < maxlen; ++i, ++offset) {
-        buffer[offset] = ": "[i];
-    }
-    for (i = 0; i < valuelen && offset < maxlen; ++i, ++offset) {
-        buffer[offset] = value[i];
-    }
-    buffer[offset] = '\0';
-    return offset;
-}
-
-
 void h2_util_camel_case_header(char *s, size_t len)
 {
     size_t start = 1;
